# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list
from typing import Optional

from fit_tool.base_type import BaseType
from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.endian import Endian
from fit_tool.field import Field
from fit_tool.profile.profile_type import *
from fit_tool.sub_field import SubField


class WorkoutStepMessage(DataMessage):
    ID = 27
    NAME = 'workout_step'

    @staticmethod
    def __get_field_size(definition_message: DefinitionMessage, field_id: int) -> int:
        size = 0
        if definition_message:
            field_definition = definition_message.get_field_definition(field_id)
            if field_definition:
                size = field_definition.size

        return size

    def __init__(self, definition_message=None, developer_fields=None, local_id: int = 0,
                 endian: Endian = Endian.LITTLE):
        super().__init__(name=WorkoutStepMessage.NAME,
                         global_id=WorkoutStepMessage.ID,
                         local_id=definition_message.local_id if definition_message else local_id,
                         endian=definition_message.endian if definition_message else endian,
                         definition_message=definition_message,
                         developer_fields=developer_fields,
                         fields=[
                             MessageIndexField(
                                 size=self.__get_field_size(definition_message, MessageIndexField.ID),
                                 growable=definition_message is None),
                             WorkoutStepWorkoutStepNameField(
                                 size=self.__get_field_size(definition_message, WorkoutStepWorkoutStepNameField.ID),
                                 growable=definition_message is None),
                             WorkoutStepDurationTypeField(
                                 size=self.__get_field_size(definition_message, WorkoutStepDurationTypeField.ID),
                                 growable=definition_message is None),
                             WorkoutStepDurationValueField(
                                 size=self.__get_field_size(definition_message, WorkoutStepDurationValueField.ID),
                                 growable=definition_message is None),
                             WorkoutStepTargetTypeField(
                                 size=self.__get_field_size(definition_message, WorkoutStepTargetTypeField.ID),
                                 growable=definition_message is None),
                             WorkoutStepTargetValueField(
                                 size=self.__get_field_size(definition_message, WorkoutStepTargetValueField.ID),
                                 growable=definition_message is None),
                             WorkoutStepCustomTargetValueLowField(
                                 size=self.__get_field_size(definition_message,
                                                            WorkoutStepCustomTargetValueLowField.ID),
                                 growable=definition_message is None),
                             WorkoutStepCustomTargetValueHighField(
                                 size=self.__get_field_size(definition_message,
                                                            WorkoutStepCustomTargetValueHighField.ID),
                                 growable=definition_message is None),
                             WorkoutStepIntensityField(
                                 size=self.__get_field_size(definition_message, WorkoutStepIntensityField.ID),
                                 growable=definition_message is None),
                             WorkoutStepNotesField(
                                 size=self.__get_field_size(definition_message, WorkoutStepNotesField.ID),
                                 growable=definition_message is None),
                             WorkoutStepEquipmentField(
                                 size=self.__get_field_size(definition_message, WorkoutStepEquipmentField.ID),
                                 growable=definition_message is None),
                             WorkoutStepExerciseCategoryField(
                                 size=self.__get_field_size(definition_message, WorkoutStepExerciseCategoryField.ID),
                                 growable=definition_message is None),
                             WorkoutStepExerciseNameField(
                                 size=self.__get_field_size(definition_message, WorkoutStepExerciseNameField.ID),
                                 growable=definition_message is None),
                             WorkoutStepExerciseWeightField(
                                 size=self.__get_field_size(definition_message, WorkoutStepExerciseWeightField.ID),
                                 growable=definition_message is None),
                             WorkoutStepWeightDisplayUnitField(
                                 size=self.__get_field_size(definition_message, WorkoutStepWeightDisplayUnitField.ID),
                                 growable=definition_message is None)
                         ])

        self.growable = self.definition_message is None

    @classmethod
    def from_bytes(cls, definition_message: DefinitionMessage, developer_fields: list[DeveloperField],
                   bytes_buffer: bytes, offset: int = 0):
        message = cls(definition_message=definition_message, developer_fields=developer_fields)
        message.read_from_bytes(bytes_buffer, offset)
        return message

    @property
    def message_index(self) -> Optional[int]:
        field = self.get_field(MessageIndexField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @message_index.setter
    def message_index(self, value: int):
        field = self.get_field(MessageIndexField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def workout_step_name(self) -> Optional[str]:
        field = self.get_field(WorkoutStepWorkoutStepNameField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @workout_step_name.setter
    def workout_step_name(self, value: str):
        field = self.get_field(WorkoutStepWorkoutStepNameField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def duration_type(self) -> Optional[WorkoutStepDuration]:
        field = self.get_field(WorkoutStepDurationTypeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @duration_type.setter
    def duration_type(self, value: WorkoutStepDuration):
        field = self.get_field(WorkoutStepDurationTypeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def duration_value(self) -> Optional[int]:
        field = self.get_field(WorkoutStepDurationValueField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @duration_value.setter
    def duration_value(self, value: int):
        field = self.get_field(WorkoutStepDurationValueField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def duration_time(self) -> Optional[float]:
        field = self.get_field(WorkoutStepDurationValueField.ID)
        type_field = self.get_field(WorkoutStepDurationTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [0, 28]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @duration_time.setter
    def duration_time(self, value: float):
        field = self.get_field(WorkoutStepDurationValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def duration_distance(self) -> Optional[float]:
        field = self.get_field(WorkoutStepDurationValueField.ID)
        type_field = self.get_field(WorkoutStepDurationTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [1]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @duration_distance.setter
    def duration_distance(self, value: float):
        field = self.get_field(WorkoutStepDurationValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def duration_hr(self) -> Optional[int]:
        field = self.get_field(WorkoutStepDurationValueField.ID)
        type_field = self.get_field(WorkoutStepDurationTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [2, 3]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @duration_hr.setter
    def duration_hr(self, value: int):
        field = self.get_field(WorkoutStepDurationValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def duration_calories(self) -> Optional[int]:
        field = self.get_field(WorkoutStepDurationValueField.ID)
        type_field = self.get_field(WorkoutStepDurationTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [4]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @duration_calories.setter
    def duration_calories(self, value: int):
        field = self.get_field(WorkoutStepDurationValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def duration_step(self) -> Optional[int]:
        field = self.get_field(WorkoutStepDurationValueField.ID)
        type_field = self.get_field(WorkoutStepDurationTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [6, 7, 8, 9, 10, 11, 12, 13]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @duration_step.setter
    def duration_step(self, value: int):
        field = self.get_field(WorkoutStepDurationValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def duration_power(self) -> Optional[int]:
        field = self.get_field(WorkoutStepDurationValueField.ID)
        type_field = self.get_field(WorkoutStepDurationTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [14, 15]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @duration_power.setter
    def duration_power(self, value: int):
        field = self.get_field(WorkoutStepDurationValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def duration_reps(self) -> Optional[int]:
        field = self.get_field(WorkoutStepDurationValueField.ID)
        type_field = self.get_field(WorkoutStepDurationTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [29]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @duration_reps.setter
    def duration_reps(self, value: int):
        field = self.get_field(WorkoutStepDurationValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def target_type(self) -> Optional[WorkoutStepTarget]:
        field = self.get_field(WorkoutStepTargetTypeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @target_type.setter
    def target_type(self, value: WorkoutStepTarget):
        field = self.get_field(WorkoutStepTargetTypeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def target_value(self) -> Optional[int]:
        field = self.get_field(WorkoutStepTargetValueField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @target_value.setter
    def target_value(self, value: int):
        field = self.get_field(WorkoutStepTargetValueField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def target_speed_zone(self) -> Optional[int]:
        field = self.get_field(WorkoutStepTargetValueField.ID)
        type_field = self.get_field(WorkoutStepTargetTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [0]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @target_speed_zone.setter
    def target_speed_zone(self, value: int):
        field = self.get_field(WorkoutStepTargetValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def target_hr_zone(self) -> Optional[int]:
        field = self.get_field(WorkoutStepTargetValueField.ID)
        type_field = self.get_field(WorkoutStepTargetTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [1]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @target_hr_zone.setter
    def target_hr_zone(self, value: int):
        field = self.get_field(WorkoutStepTargetValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def target_cadence_zone(self) -> Optional[int]:
        field = self.get_field(WorkoutStepTargetValueField.ID)
        type_field = self.get_field(WorkoutStepTargetTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [3]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @target_cadence_zone.setter
    def target_cadence_zone(self, value: int):
        field = self.get_field(WorkoutStepTargetValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def target_power_zone(self) -> Optional[int]:
        field = self.get_field(WorkoutStepTargetValueField.ID)
        type_field = self.get_field(WorkoutStepTargetTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [4]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @target_power_zone.setter
    def target_power_zone(self, value: int):
        field = self.get_field(WorkoutStepTargetValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def target_repeat_steps(self) -> Optional[int]:
        field = self.get_field(WorkoutStepTargetValueField.ID)
        type_field = self.get_field(WorkoutStepDurationTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [6]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @target_repeat_steps.setter
    def target_repeat_steps(self, value: int):
        field = self.get_field(WorkoutStepTargetValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def target_repeat_time(self) -> Optional[float]:
        field = self.get_field(WorkoutStepTargetValueField.ID)
        type_field = self.get_field(WorkoutStepDurationTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [7]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @target_repeat_time.setter
    def target_repeat_time(self, value: float):
        field = self.get_field(WorkoutStepTargetValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def target_repeat_distance(self) -> Optional[float]:
        field = self.get_field(WorkoutStepTargetValueField.ID)
        type_field = self.get_field(WorkoutStepDurationTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [8]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @target_repeat_distance.setter
    def target_repeat_distance(self, value: float):
        field = self.get_field(WorkoutStepTargetValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def target_repeat_calories(self) -> Optional[int]:
        field = self.get_field(WorkoutStepTargetValueField.ID)
        type_field = self.get_field(WorkoutStepDurationTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [9]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @target_repeat_calories.setter
    def target_repeat_calories(self, value: int):
        field = self.get_field(WorkoutStepTargetValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def target_repeat_hr(self) -> Optional[int]:
        field = self.get_field(WorkoutStepTargetValueField.ID)
        type_field = self.get_field(WorkoutStepDurationTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [10, 11]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @target_repeat_hr.setter
    def target_repeat_hr(self, value: int):
        field = self.get_field(WorkoutStepTargetValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def target_repeat_power(self) -> Optional[int]:
        field = self.get_field(WorkoutStepTargetValueField.ID)
        type_field = self.get_field(WorkoutStepDurationTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [12, 13]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @target_repeat_power.setter
    def target_repeat_power(self, value: int):
        field = self.get_field(WorkoutStepTargetValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def target_stroke_type(self) -> Optional[SwimStroke]:
        field = self.get_field(WorkoutStepTargetValueField.ID)
        type_field = self.get_field(WorkoutStepTargetTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [11]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @target_stroke_type.setter
    def target_stroke_type(self, value: SwimStroke):
        field = self.get_field(WorkoutStepTargetValueField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def custom_target_value_low(self) -> Optional[int]:
        field = self.get_field(WorkoutStepCustomTargetValueLowField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @custom_target_value_low.setter
    def custom_target_value_low(self, value: int):
        field = self.get_field(WorkoutStepCustomTargetValueLowField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def custom_target_speed_low(self) -> Optional[float]:
        field = self.get_field(WorkoutStepCustomTargetValueLowField.ID)
        type_field = self.get_field(WorkoutStepTargetTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [0, 12]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @custom_target_speed_low.setter
    def custom_target_speed_low(self, value: float):
        field = self.get_field(WorkoutStepCustomTargetValueLowField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def custom_target_heart_rate_low(self) -> Optional[int]:
        field = self.get_field(WorkoutStepCustomTargetValueLowField.ID)
        type_field = self.get_field(WorkoutStepTargetTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [1, 13]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @custom_target_heart_rate_low.setter
    def custom_target_heart_rate_low(self, value: int):
        field = self.get_field(WorkoutStepCustomTargetValueLowField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def custom_target_cadence_low(self) -> Optional[int]:
        field = self.get_field(WorkoutStepCustomTargetValueLowField.ID)
        type_field = self.get_field(WorkoutStepTargetTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [3]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @custom_target_cadence_low.setter
    def custom_target_cadence_low(self, value: int):
        field = self.get_field(WorkoutStepCustomTargetValueLowField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def custom_target_power_low(self) -> Optional[int]:
        field = self.get_field(WorkoutStepCustomTargetValueLowField.ID)
        type_field = self.get_field(WorkoutStepTargetTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [4, 7, 8, 9, 10]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @custom_target_power_low.setter
    def custom_target_power_low(self, value: int):
        field = self.get_field(WorkoutStepCustomTargetValueLowField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def custom_target_value_high(self) -> Optional[int]:
        field = self.get_field(WorkoutStepCustomTargetValueHighField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @custom_target_value_high.setter
    def custom_target_value_high(self, value: int):
        field = self.get_field(WorkoutStepCustomTargetValueHighField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def custom_target_speed_high(self) -> Optional[float]:
        field = self.get_field(WorkoutStepCustomTargetValueHighField.ID)
        type_field = self.get_field(WorkoutStepTargetTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [0, 12]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @custom_target_speed_high.setter
    def custom_target_speed_high(self, value: float):
        field = self.get_field(WorkoutStepCustomTargetValueHighField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def custom_target_heart_rate_high(self) -> Optional[int]:
        field = self.get_field(WorkoutStepCustomTargetValueHighField.ID)
        type_field = self.get_field(WorkoutStepTargetTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [1, 13]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @custom_target_heart_rate_high.setter
    def custom_target_heart_rate_high(self, value: int):
        field = self.get_field(WorkoutStepCustomTargetValueHighField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def custom_target_cadence_high(self) -> Optional[int]:
        field = self.get_field(WorkoutStepCustomTargetValueHighField.ID)
        type_field = self.get_field(WorkoutStepTargetTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [3]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @custom_target_cadence_high.setter
    def custom_target_cadence_high(self, value: int):
        field = self.get_field(WorkoutStepCustomTargetValueHighField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def custom_target_power_high(self) -> Optional[int]:
        field = self.get_field(WorkoutStepCustomTargetValueHighField.ID)
        type_field = self.get_field(WorkoutStepTargetTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [4, 7, 8, 9, 10]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @custom_target_power_high.setter
    def custom_target_power_high(self, value: int):
        field = self.get_field(WorkoutStepCustomTargetValueHighField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def intensity(self) -> Optional[Intensity]:
        field = self.get_field(WorkoutStepIntensityField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @intensity.setter
    def intensity(self, value: Intensity):
        field = self.get_field(WorkoutStepIntensityField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def notes(self) -> Optional[str]:
        field = self.get_field(WorkoutStepNotesField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @notes.setter
    def notes(self, value: str):
        field = self.get_field(WorkoutStepNotesField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def equipment(self) -> Optional[WorkoutEquipment]:
        field = self.get_field(WorkoutStepEquipmentField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @equipment.setter
    def equipment(self, value: WorkoutEquipment):
        field = self.get_field(WorkoutStepEquipmentField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def exercise_category(self) -> Optional[int]:
        field = self.get_field(WorkoutStepExerciseCategoryField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @exercise_category.setter
    def exercise_category(self, value: int):
        field = self.get_field(WorkoutStepExerciseCategoryField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def exercise_name(self) -> Optional[int]:
        field = self.get_field(WorkoutStepExerciseNameField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @exercise_name.setter
    def exercise_name(self, value: int):
        field = self.get_field(WorkoutStepExerciseNameField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def exercise_weight(self) -> Optional[float]:
        field = self.get_field(WorkoutStepExerciseWeightField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @exercise_weight.setter
    def exercise_weight(self, value: float):
        field = self.get_field(WorkoutStepExerciseWeightField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def weight_display_unit(self) -> Optional[int]:
        field = self.get_field(WorkoutStepWeightDisplayUnitField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @weight_display_unit.setter
    def weight_display_unit(self, value: int):
        field = self.get_field(WorkoutStepWeightDisplayUnitField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)


class MessageIndexField(Field):
    ID = 254

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='message_index',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class WorkoutStepWorkoutStepNameField(Field):
    ID = 0

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='wkt_step_name',
            field_id=self.ID,
            base_type=BaseType.STRING,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class WorkoutStepDurationTypeField(Field):
    ID = 1

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='duration_type',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class WorkoutStepDurationValueField(Field):
    ID = 2

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='duration_value',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
                SubField(
                    name='duration_time',
                    base_type=BaseType.UINT32,
                    scale=1000,
                    offset=0,
                    units='s',
                    reference_map={
                        WorkoutStepDurationTypeField.ID: [0, 28]
                    }),
                SubField(
                    name='duration_distance',
                    base_type=BaseType.UINT32,
                    scale=100,
                    offset=0,
                    units='m',
                    reference_map={
                        WorkoutStepDurationTypeField.ID: [1]
                    }),
                SubField(
                    name='duration_hr',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    units='% or bpm',
                    reference_map={
                        WorkoutStepDurationTypeField.ID: [2, 3]
                    }),
                SubField(
                    name='duration_calories',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    units='calories',
                    reference_map={
                        WorkoutStepDurationTypeField.ID: [4]
                    }),
                SubField(
                    name='duration_step',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    reference_map={
                        WorkoutStepDurationTypeField.ID: [6, 7, 8, 9, 10, 11, 12, 13]
                    }),
                SubField(
                    name='duration_power',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    units='% or watts',
                    reference_map={
                        WorkoutStepDurationTypeField.ID: [14, 15]
                    }),
                SubField(
                    name='duration_reps',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    reference_map={
                        WorkoutStepDurationTypeField.ID: [29]
                    })
            ]
        )


class WorkoutStepTargetTypeField(Field):
    ID = 3

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='target_type',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class WorkoutStepTargetValueField(Field):
    ID = 4

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='target_value',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
                SubField(
                    name='target_speed_zone',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    reference_map={
                        WorkoutStepTargetTypeField.ID: [0]
                    }),
                SubField(
                    name='target_hr_zone',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    reference_map={
                        WorkoutStepTargetTypeField.ID: [1]
                    }),
                SubField(
                    name='target_cadence_zone',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    reference_map={
                        WorkoutStepTargetTypeField.ID: [3]
                    }),
                SubField(
                    name='target_power_zone',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    reference_map={
                        WorkoutStepTargetTypeField.ID: [4]
                    }),
                SubField(
                    name='repeat_steps',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    reference_map={
                        WorkoutStepDurationTypeField.ID: [6]
                    }),
                SubField(
                    name='repeat_time',
                    base_type=BaseType.UINT32,
                    scale=1000,
                    offset=0,
                    units='s',
                    reference_map={
                        WorkoutStepDurationTypeField.ID: [7]
                    }),
                SubField(
                    name='repeat_distance',
                    base_type=BaseType.UINT32,
                    scale=100,
                    offset=0,
                    units='m',
                    reference_map={
                        WorkoutStepDurationTypeField.ID: [8]
                    }),
                SubField(
                    name='repeat_calories',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    units='calories',
                    reference_map={
                        WorkoutStepDurationTypeField.ID: [9]
                    }),
                SubField(
                    name='repeat_hr',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    units='% or bpm',
                    reference_map={
                        WorkoutStepDurationTypeField.ID: [10, 11]
                    }),
                SubField(
                    name='repeat_power',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    units='% or watts',
                    reference_map={
                        WorkoutStepDurationTypeField.ID: [12, 13]
                    }),
                SubField(
                    name='target_stroke_type',
                    base_type=BaseType.ENUM,
                    scale=1,
                    offset=0,
                    reference_map={
                        WorkoutStepTargetTypeField.ID: [11]
                    })
            ]
        )


class WorkoutStepCustomTargetValueLowField(Field):
    ID = 5

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='custom_target_value_low',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
                SubField(
                    name='custom_target_speed_low',
                    base_type=BaseType.UINT32,
                    scale=1000,
                    offset=0,
                    units='m/s',
                    reference_map={
                        WorkoutStepTargetTypeField.ID: [0, 12]
                    }),
                SubField(
                    name='custom_target_heart_rate_low',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    units='% or bpm',
                    reference_map={
                        WorkoutStepTargetTypeField.ID: [1, 13]
                    }),
                SubField(
                    name='custom_target_cadence_low',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    units='rpm',
                    reference_map={
                        WorkoutStepTargetTypeField.ID: [3]
                    }),
                SubField(
                    name='custom_target_power_low',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    units='% or watts',
                    reference_map={
                        WorkoutStepTargetTypeField.ID: [4, 7, 8, 9, 10]
                    })
            ]
        )


class WorkoutStepCustomTargetValueHighField(Field):
    ID = 6

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='custom_target_value_high',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
                SubField(
                    name='custom_target_speed_high',
                    base_type=BaseType.UINT32,
                    scale=1000,
                    offset=0,
                    units='m/s',
                    reference_map={
                        WorkoutStepTargetTypeField.ID: [0, 12]
                    }),
                SubField(
                    name='custom_target_heart_rate_high',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    units='% or bpm',
                    reference_map={
                        WorkoutStepTargetTypeField.ID: [1, 13]
                    }),
                SubField(
                    name='custom_target_cadence_high',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    units='rpm',
                    reference_map={
                        WorkoutStepTargetTypeField.ID: [3]
                    }),
                SubField(
                    name='custom_target_power_high',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    units='% or watts',
                    reference_map={
                        WorkoutStepTargetTypeField.ID: [4, 7, 8, 9, 10]
                    })
            ]
        )


class WorkoutStepIntensityField(Field):
    ID = 7

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='intensity',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class WorkoutStepNotesField(Field):
    ID = 8

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='notes',
            field_id=self.ID,
            base_type=BaseType.STRING,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class WorkoutStepEquipmentField(Field):
    ID = 9

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='equipment',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class WorkoutStepExerciseCategoryField(Field):
    ID = 10

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='exercise_category',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class WorkoutStepExerciseNameField(Field):
    ID = 11

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='exercise_name',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class WorkoutStepExerciseWeightField(Field):
    ID = 12

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='exercise_weight',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=100,
            size=size,
            units='kg',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class WorkoutStepWeightDisplayUnitField(Field):
    ID = 13

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='weight_display_unit',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )



from typing import List as list

from fit_tool.base_type import BaseType
from fit_tool.field import Field
from fit_tool.field_component import FieldComponent
from fit_tool.sub_field import SubField


class DeveloperField(Field):
    def __init__(self, field_id: int = 0, name: str = '', base_type: BaseType = BaseType.ENUM, offset: float = None,
                 scale: float = None, units: str = '', is_accumulated: bool = False,
                 is_expanded_field=False, sub_fields: list[SubField] = None, components: list[FieldComponent] = None,
                 size: int = 0, growable: bool = False, type_name: str = '', developer_data_index: int = 0):
        super().__init__(field_id=field_id, name=name, base_type=base_type, offset=offset, scale=scale,
                         units=units, is_accumulated=is_accumulated, is_expanded_field=is_expanded_field,
                         sub_fields=sub_fields, components=components, size=size, growable=growable,
                         type_name=type_name)
        self.developer_data_index = developer_data_index

    @classmethod
    def from_developer_field(cls, other, size: int = None):
        size_ = size if size is not None else other.size
        return cls(field_id=other.field_id, name=other.name, base_type=other.base_type, offset=other.offset,
                   scale=other.scale, units=other.units, is_accumulated=other.is_accumulated,
                   is_expanded_field=other.is_expanded_field, sub_fields=other.sub_fields, components=other.components,
                   size=size_, growable=other.growable, type_name=other.type_name,
                   developer_data_index=other.developer_data_index)



# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list
from typing import Optional

from fit_tool.base_type import BaseType
from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.endian import Endian
from fit_tool.field import Field
from fit_tool.profile.profile_type import *
from fit_tool.sub_field import SubField


class MonitoringMessage(DataMessage):
    ID = 55
    NAME = 'monitoring'

    @staticmethod
    def __get_field_size(definition_message: DefinitionMessage, field_id: int) -> int:
        size = 0
        if definition_message:
            field_definition = definition_message.get_field_definition(field_id)
            if field_definition:
                size = field_definition.size

        return size

    def __init__(self, definition_message=None, developer_fields=None, local_id: int = 0,
                 endian: Endian = Endian.LITTLE):
        super().__init__(name=MonitoringMessage.NAME,
                         global_id=MonitoringMessage.ID,
                         local_id=definition_message.local_id if definition_message else local_id,
                         endian=definition_message.endian if definition_message else endian,
                         definition_message=definition_message,
                         developer_fields=developer_fields,
                         fields=[
                             TimestampField(
                                 size=self.__get_field_size(definition_message, TimestampField.ID),
                                 growable=definition_message is None),
                             MonitoringDeviceIndexField(
                                 size=self.__get_field_size(definition_message, MonitoringDeviceIndexField.ID),
                                 growable=definition_message is None),
                             MonitoringCaloriesField(
                                 size=self.__get_field_size(definition_message, MonitoringCaloriesField.ID),
                                 growable=definition_message is None),
                             MonitoringDistanceField(
                                 size=self.__get_field_size(definition_message, MonitoringDistanceField.ID),
                                 growable=definition_message is None),
                             MonitoringCyclesField(
                                 size=self.__get_field_size(definition_message, MonitoringCyclesField.ID),
                                 growable=definition_message is None),
                             MonitoringActiveTimeField(
                                 size=self.__get_field_size(definition_message, MonitoringActiveTimeField.ID),
                                 growable=definition_message is None),
                             MonitoringActivityTypeField(
                                 size=self.__get_field_size(definition_message, MonitoringActivityTypeField.ID),
                                 growable=definition_message is None),
                             MonitoringActivitySubtypeField(
                                 size=self.__get_field_size(definition_message, MonitoringActivitySubtypeField.ID),
                                 growable=definition_message is None),
                             MonitoringActivityLevelField(
                                 size=self.__get_field_size(definition_message, MonitoringActivityLevelField.ID),
                                 growable=definition_message is None),
                             MonitoringDistance16Field(
                                 size=self.__get_field_size(definition_message, MonitoringDistance16Field.ID),
                                 growable=definition_message is None),
                             MonitoringCycles16Field(
                                 size=self.__get_field_size(definition_message, MonitoringCycles16Field.ID),
                                 growable=definition_message is None),
                             MonitoringActiveTime16Field(
                                 size=self.__get_field_size(definition_message, MonitoringActiveTime16Field.ID),
                                 growable=definition_message is None),
                             MonitoringLocalTimestampField(
                                 size=self.__get_field_size(definition_message, MonitoringLocalTimestampField.ID),
                                 growable=definition_message is None),
                             MonitoringTemperatureField(
                                 size=self.__get_field_size(definition_message, MonitoringTemperatureField.ID),
                                 growable=definition_message is None),
                             MonitoringTemperatureMinField(
                                 size=self.__get_field_size(definition_message, MonitoringTemperatureMinField.ID),
                                 growable=definition_message is None),
                             MonitoringTemperatureMaxField(
                                 size=self.__get_field_size(definition_message, MonitoringTemperatureMaxField.ID),
                                 growable=definition_message is None),
                             MonitoringActivityTimeField(
                                 size=self.__get_field_size(definition_message, MonitoringActivityTimeField.ID),
                                 growable=definition_message is None),
                             MonitoringActiveCaloriesField(
                                 size=self.__get_field_size(definition_message, MonitoringActiveCaloriesField.ID),
                                 growable=definition_message is None),
                             MonitoringCurrentActivityTypeIntensityField(
                                 size=self.__get_field_size(definition_message,
                                                            MonitoringCurrentActivityTypeIntensityField.ID),
                                 growable=definition_message is None),
                             MonitoringTimestampMin8Field(
                                 size=self.__get_field_size(definition_message, MonitoringTimestampMin8Field.ID),
                                 growable=definition_message is None),
                             MonitoringTimestamp16Field(
                                 size=self.__get_field_size(definition_message, MonitoringTimestamp16Field.ID),
                                 growable=definition_message is None),
                             MonitoringHeartRateField(
                                 size=self.__get_field_size(definition_message, MonitoringHeartRateField.ID),
                                 growable=definition_message is None),
                             MonitoringIntensityField(
                                 size=self.__get_field_size(definition_message, MonitoringIntensityField.ID),
                                 growable=definition_message is None),
                             MonitoringDurationMinField(
                                 size=self.__get_field_size(definition_message, MonitoringDurationMinField.ID),
                                 growable=definition_message is None),
                             MonitoringDurationField(
                                 size=self.__get_field_size(definition_message, MonitoringDurationField.ID),
                                 growable=definition_message is None),
                             MonitoringAscentField(
                                 size=self.__get_field_size(definition_message, MonitoringAscentField.ID),
                                 growable=definition_message is None),
                             MonitoringDescentField(
                                 size=self.__get_field_size(definition_message, MonitoringDescentField.ID),
                                 growable=definition_message is None),
                             MonitoringModerateActivityMinutesField(
                                 size=self.__get_field_size(definition_message,
                                                            MonitoringModerateActivityMinutesField.ID),
                                 growable=definition_message is None),
                             MonitoringVigorousActivityMinutesField(
                                 size=self.__get_field_size(definition_message,
                                                            MonitoringVigorousActivityMinutesField.ID),
                                 growable=definition_message is None)
                         ])

        self.growable = self.definition_message is None

    @classmethod
    def from_bytes(cls, definition_message: DefinitionMessage, developer_fields: list[DeveloperField],
                   bytes_buffer: bytes, offset: int = 0):
        message = cls(definition_message=definition_message, developer_fields=developer_fields)
        message.read_from_bytes(bytes_buffer, offset)
        return message

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @property
    def timestamp(self) -> Optional[int]:
        field = self.get_field(TimestampField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @timestamp.setter
    def timestamp(self, value: int):
        field = self.get_field(TimestampField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def device_index(self) -> Optional[int]:
        field = self.get_field(MonitoringDeviceIndexField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @device_index.setter
    def device_index(self, value: int):
        field = self.get_field(MonitoringDeviceIndexField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def calories(self) -> Optional[int]:
        field = self.get_field(MonitoringCaloriesField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @calories.setter
    def calories(self, value: int):
        field = self.get_field(MonitoringCaloriesField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def distance(self) -> Optional[float]:
        field = self.get_field(MonitoringDistanceField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @distance.setter
    def distance(self, value: float):
        field = self.get_field(MonitoringDistanceField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def cycles(self) -> Optional[float]:
        field = self.get_field(MonitoringCyclesField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @cycles.setter
    def cycles(self, value: float):
        field = self.get_field(MonitoringCyclesField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def steps(self) -> Optional[int]:
        field = self.get_field(MonitoringCyclesField.ID)
        type_field = self.get_field(MonitoringActivityTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [6, 1]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @steps.setter
    def steps(self, value: int):
        field = self.get_field(MonitoringCyclesField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def strokes(self) -> Optional[float]:
        field = self.get_field(MonitoringCyclesField.ID)
        type_field = self.get_field(MonitoringActivityTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [2, 5]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @strokes.setter
    def strokes(self, value: float):
        field = self.get_field(MonitoringCyclesField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def active_time(self) -> Optional[float]:
        field = self.get_field(MonitoringActiveTimeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @active_time.setter
    def active_time(self, value: float):
        field = self.get_field(MonitoringActiveTimeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def activity_type(self) -> Optional[ActivityType]:
        field = self.get_field(MonitoringActivityTypeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @activity_type.setter
    def activity_type(self, value: ActivityType):
        field = self.get_field(MonitoringActivityTypeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def activity_subtype(self) -> Optional[ActivitySubtype]:
        field = self.get_field(MonitoringActivitySubtypeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @activity_subtype.setter
    def activity_subtype(self, value: ActivitySubtype):
        field = self.get_field(MonitoringActivitySubtypeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def activity_level(self) -> Optional[ActivityLevel]:
        field = self.get_field(MonitoringActivityLevelField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @activity_level.setter
    def activity_level(self, value: ActivityLevel):
        field = self.get_field(MonitoringActivityLevelField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def distance_16(self) -> Optional[int]:
        field = self.get_field(MonitoringDistance16Field.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @distance_16.setter
    def distance_16(self, value: int):
        field = self.get_field(MonitoringDistance16Field.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def cycles_16(self) -> Optional[int]:
        field = self.get_field(MonitoringCycles16Field.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @cycles_16.setter
    def cycles_16(self, value: int):
        field = self.get_field(MonitoringCycles16Field.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def active_time_16(self) -> Optional[int]:
        field = self.get_field(MonitoringActiveTime16Field.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @active_time_16.setter
    def active_time_16(self, value: int):
        field = self.get_field(MonitoringActiveTime16Field.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def local_timestamp(self) -> Optional[int]:
        field = self.get_field(MonitoringLocalTimestampField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @local_timestamp.setter
    def local_timestamp(self, value: int):
        field = self.get_field(MonitoringLocalTimestampField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def temperature(self) -> Optional[float]:
        field = self.get_field(MonitoringTemperatureField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @temperature.setter
    def temperature(self, value: float):
        field = self.get_field(MonitoringTemperatureField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def temperature_min(self) -> Optional[float]:
        field = self.get_field(MonitoringTemperatureMinField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @temperature_min.setter
    def temperature_min(self, value: float):
        field = self.get_field(MonitoringTemperatureMinField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def temperature_max(self) -> Optional[float]:
        field = self.get_field(MonitoringTemperatureMaxField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @temperature_max.setter
    def temperature_max(self, value: float):
        field = self.get_field(MonitoringTemperatureMaxField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def activity_time(self) -> Optional[list[int]]:
        field = self.get_field(MonitoringActivityTimeField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @activity_time.setter
    def activity_time(self, value: list[int]):
        field = self.get_field(MonitoringActivityTimeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def active_calories(self) -> Optional[int]:
        field = self.get_field(MonitoringActiveCaloriesField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @active_calories.setter
    def active_calories(self, value: int):
        field = self.get_field(MonitoringActiveCaloriesField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def current_activity_type_intensity(self) -> Optional[int]:
        field = self.get_field(MonitoringCurrentActivityTypeIntensityField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @current_activity_type_intensity.setter
    def current_activity_type_intensity(self, value: int):
        field = self.get_field(MonitoringCurrentActivityTypeIntensityField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def timestamp_min_8(self) -> Optional[int]:
        field = self.get_field(MonitoringTimestampMin8Field.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @timestamp_min_8.setter
    def timestamp_min_8(self, value: int):
        field = self.get_field(MonitoringTimestampMin8Field.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def timestamp_16(self) -> Optional[int]:
        field = self.get_field(MonitoringTimestamp16Field.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @timestamp_16.setter
    def timestamp_16(self, value: int):
        field = self.get_field(MonitoringTimestamp16Field.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def heart_rate(self) -> Optional[int]:
        field = self.get_field(MonitoringHeartRateField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @heart_rate.setter
    def heart_rate(self, value: int):
        field = self.get_field(MonitoringHeartRateField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def intensity(self) -> Optional[float]:
        field = self.get_field(MonitoringIntensityField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @intensity.setter
    def intensity(self, value: float):
        field = self.get_field(MonitoringIntensityField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def duration_min(self) -> Optional[int]:
        field = self.get_field(MonitoringDurationMinField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @duration_min.setter
    def duration_min(self, value: int):
        field = self.get_field(MonitoringDurationMinField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def duration(self) -> Optional[int]:
        field = self.get_field(MonitoringDurationField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @duration.setter
    def duration(self, value: int):
        field = self.get_field(MonitoringDurationField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def ascent(self) -> Optional[float]:
        field = self.get_field(MonitoringAscentField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @ascent.setter
    def ascent(self, value: float):
        field = self.get_field(MonitoringAscentField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def descent(self) -> Optional[float]:
        field = self.get_field(MonitoringDescentField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @descent.setter
    def descent(self, value: float):
        field = self.get_field(MonitoringDescentField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def moderate_activity_minutes(self) -> Optional[int]:
        field = self.get_field(MonitoringModerateActivityMinutesField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @moderate_activity_minutes.setter
    def moderate_activity_minutes(self, value: int):
        field = self.get_field(MonitoringModerateActivityMinutesField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def vigorous_activity_minutes(self) -> Optional[int]:
        field = self.get_field(MonitoringVigorousActivityMinutesField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @vigorous_activity_minutes.setter
    def vigorous_activity_minutes(self, value: int):
        field = self.get_field(MonitoringVigorousActivityMinutesField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)


class TimestampField(Field):
    ID = 253

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='timestamp',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=-631065600000,
            scale=0.001,
            size=size,
            units='ms',
            type_name='date_time',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringDeviceIndexField(Field):
    ID = 0

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='device_index',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringCaloriesField(Field):
    ID = 1

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='calories',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='kcal',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringDistanceField(Field):
    ID = 2

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='distance',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=100,
            size=size,
            units='m',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringCyclesField(Field):
    ID = 3

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='cycles',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=2,
            size=size,
            units='cycles',
            type_name='',
            growable=growable,
            sub_fields=[
                SubField(
                    name='steps',
                    base_type=BaseType.UINT32,
                    scale=1,
                    offset=0,
                    units='steps',
                    reference_map={
                        MonitoringActivityTypeField.ID: [6, 1]
                    }),
                SubField(
                    name='strokes',
                    base_type=BaseType.UINT32,
                    scale=2,
                    offset=0,
                    units='strokes',
                    reference_map={
                        MonitoringActivityTypeField.ID: [2, 5]
                    })
            ]
        )


class MonitoringActiveTimeField(Field):
    ID = 4

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='active_time',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1000,
            size=size,
            units='s',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringActivityTypeField(Field):
    ID = 5

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='activity_type',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringActivitySubtypeField(Field):
    ID = 6

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='activity_subtype',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringActivityLevelField(Field):
    ID = 7

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='activity_level',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringDistance16Field(Field):
    ID = 8

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='distance_16',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='100 * m',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringCycles16Field(Field):
    ID = 9

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='cycles_16',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='2 * cycles (steps)',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringActiveTime16Field(Field):
    ID = 10

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='active_time_16',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='s',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringLocalTimestampField(Field):
    ID = 11

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='local_timestamp',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringTemperatureField(Field):
    ID = 12

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='temperature',
            field_id=self.ID,
            base_type=BaseType.SINT16,
            offset=0,
            scale=100,
            size=size,
            units='C',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringTemperatureMinField(Field):
    ID = 14

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='temperature_min',
            field_id=self.ID,
            base_type=BaseType.SINT16,
            offset=0,
            scale=100,
            size=size,
            units='C',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringTemperatureMaxField(Field):
    ID = 15

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='temperature_max',
            field_id=self.ID,
            base_type=BaseType.SINT16,
            offset=0,
            scale=100,
            size=size,
            units='C',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringActivityTimeField(Field):
    ID = 16

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='activity_time',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='minutes',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringActiveCaloriesField(Field):
    ID = 19

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='active_calories',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='kcal',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringCurrentActivityTypeIntensityField(Field):
    ID = 24

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='current_activity_type_intensity',
            field_id=self.ID,
            base_type=BaseType.BYTE,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringTimestampMin8Field(Field):
    ID = 25

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='timestamp_min_8',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            units='min',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringTimestamp16Field(Field):
    ID = 26

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='timestamp_16',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='s',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringHeartRateField(Field):
    ID = 27

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='heart_rate',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            units='bpm',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringIntensityField(Field):
    ID = 28

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='intensity',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=10,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringDurationMinField(Field):
    ID = 29

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='duration_min',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='min',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringDurationField(Field):
    ID = 30

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='duration',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            units='s',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringAscentField(Field):
    ID = 31

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='ascent',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1000,
            size=size,
            units='m',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringDescentField(Field):
    ID = 32

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='descent',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1000,
            size=size,
            units='m',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringModerateActivityMinutesField(Field):
    ID = 33

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='moderate_activity_minutes',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='minutes',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MonitoringVigorousActivityMinutesField(Field):
    ID = 34

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='vigorous_activity_minutes',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='minutes',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )



# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list
from typing import Optional

from fit_tool.base_type import BaseType
from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.endian import Endian
from fit_tool.field import Field


class MagnetometerDataMessage(DataMessage):
    ID = 208
    NAME = 'magnetometer_data'

    @staticmethod
    def __get_field_size(definition_message: DefinitionMessage, field_id: int) -> int:
        size = 0
        if definition_message:
            field_definition = definition_message.get_field_definition(field_id)
            if field_definition:
                size = field_definition.size

        return size

    def __init__(self, definition_message=None, developer_fields=None, local_id: int = 0,
                 endian: Endian = Endian.LITTLE):
        super().__init__(name=MagnetometerDataMessage.NAME,
                         global_id=MagnetometerDataMessage.ID,
                         local_id=definition_message.local_id if definition_message else local_id,
                         endian=definition_message.endian if definition_message else endian,
                         definition_message=definition_message,
                         developer_fields=developer_fields,
                         fields=[
                             TimestampField(
                                 size=self.__get_field_size(definition_message, TimestampField.ID),
                                 growable=definition_message is None),
                             MagnetometerDataTimestampMsField(
                                 size=self.__get_field_size(definition_message, MagnetometerDataTimestampMsField.ID),
                                 growable=definition_message is None),
                             MagnetometerDataSampleTimeOffsetField(
                                 size=self.__get_field_size(definition_message,
                                                            MagnetometerDataSampleTimeOffsetField.ID),
                                 growable=definition_message is None),
                             MagnetometerDataMagXField(
                                 size=self.__get_field_size(definition_message, MagnetometerDataMagXField.ID),
                                 growable=definition_message is None),
                             MagnetometerDataMagYField(
                                 size=self.__get_field_size(definition_message, MagnetometerDataMagYField.ID),
                                 growable=definition_message is None),
                             MagnetometerDataMagZField(
                                 size=self.__get_field_size(definition_message, MagnetometerDataMagZField.ID),
                                 growable=definition_message is None),
                             MagnetometerDataCalibratedMagXField(
                                 size=self.__get_field_size(definition_message, MagnetometerDataCalibratedMagXField.ID),
                                 growable=definition_message is None),
                             MagnetometerDataCalibratedMagYField(
                                 size=self.__get_field_size(definition_message, MagnetometerDataCalibratedMagYField.ID),
                                 growable=definition_message is None),
                             MagnetometerDataCalibratedMagZField(
                                 size=self.__get_field_size(definition_message, MagnetometerDataCalibratedMagZField.ID),
                                 growable=definition_message is None)
                         ])

        self.growable = self.definition_message is None

    @classmethod
    def from_bytes(cls, definition_message: DefinitionMessage, developer_fields: list[DeveloperField],
                   bytes_buffer: bytes, offset: int = 0):
        message = cls(definition_message=definition_message, developer_fields=developer_fields)
        message.read_from_bytes(bytes_buffer, offset)
        return message

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @property
    def timestamp(self) -> Optional[int]:
        field = self.get_field(TimestampField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @timestamp.setter
    def timestamp(self, value: int):
        field = self.get_field(TimestampField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def timestamp_ms(self) -> Optional[int]:
        field = self.get_field(MagnetometerDataTimestampMsField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @timestamp_ms.setter
    def timestamp_ms(self, value: int):
        field = self.get_field(MagnetometerDataTimestampMsField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def sample_time_offset(self) -> Optional[list[int]]:
        field = self.get_field(MagnetometerDataSampleTimeOffsetField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @sample_time_offset.setter
    def sample_time_offset(self, value: list[int]):
        field = self.get_field(MagnetometerDataSampleTimeOffsetField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def mag_x(self) -> Optional[list[int]]:
        field = self.get_field(MagnetometerDataMagXField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @mag_x.setter
    def mag_x(self, value: list[int]):
        field = self.get_field(MagnetometerDataMagXField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def mag_y(self) -> Optional[list[int]]:
        field = self.get_field(MagnetometerDataMagYField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @mag_y.setter
    def mag_y(self, value: list[int]):
        field = self.get_field(MagnetometerDataMagYField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def mag_z(self) -> Optional[list[int]]:
        field = self.get_field(MagnetometerDataMagZField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @mag_z.setter
    def mag_z(self, value: list[int]):
        field = self.get_field(MagnetometerDataMagZField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def calibrated_mag_x(self) -> Optional[list[float]]:
        field = self.get_field(MagnetometerDataCalibratedMagXField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @calibrated_mag_x.setter
    def calibrated_mag_x(self, value: list[float]):
        field = self.get_field(MagnetometerDataCalibratedMagXField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def calibrated_mag_y(self) -> Optional[list[float]]:
        field = self.get_field(MagnetometerDataCalibratedMagYField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @calibrated_mag_y.setter
    def calibrated_mag_y(self, value: list[float]):
        field = self.get_field(MagnetometerDataCalibratedMagYField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def calibrated_mag_z(self) -> Optional[list[float]]:
        field = self.get_field(MagnetometerDataCalibratedMagZField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @calibrated_mag_z.setter
    def calibrated_mag_z(self, value: list[float]):
        field = self.get_field(MagnetometerDataCalibratedMagZField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)


class TimestampField(Field):
    ID = 253

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='timestamp',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=-631065600000,
            scale=0.001,
            size=size,
            units='ms',
            type_name='date_time',
            growable=growable,
            sub_fields=[
            ]
        )


class MagnetometerDataTimestampMsField(Field):
    ID = 0

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='timestamp_ms',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='ms',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MagnetometerDataSampleTimeOffsetField(Field):
    ID = 1

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='sample_time_offset',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='ms',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MagnetometerDataMagXField(Field):
    ID = 2

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='mag_x',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='counts',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MagnetometerDataMagYField(Field):
    ID = 3

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='mag_y',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='counts',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MagnetometerDataMagZField(Field):
    ID = 4

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='mag_z',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='counts',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MagnetometerDataCalibratedMagXField(Field):
    ID = 5

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='calibrated_mag_x',
            field_id=self.ID,
            base_type=BaseType.FLOAT32,
            offset=0,
            scale=1,
            size=size,
            units='G',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MagnetometerDataCalibratedMagYField(Field):
    ID = 6

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='calibrated_mag_y',
            field_id=self.ID,
            base_type=BaseType.FLOAT32,
            offset=0,
            scale=1,
            size=size,
            units='G',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class MagnetometerDataCalibratedMagZField(Field):
    ID = 7

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='calibrated_mag_z',
            field_id=self.ID,
            base_type=BaseType.FLOAT32,
            offset=0,
            scale=1,
            size=size,
            units='G',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )



# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list
from typing import Optional

from fit_tool.base_type import BaseType
from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.endian import Endian
from fit_tool.field import Field
from fit_tool.profile.profile_type import *
from fit_tool.sub_field import SubField


class DeviceInfoMessage(DataMessage):
    ID = 23
    NAME = 'device_info'

    @staticmethod
    def __get_field_size(definition_message: DefinitionMessage, field_id: int) -> int:
        size = 0
        if definition_message:
            field_definition = definition_message.get_field_definition(field_id)
            if field_definition:
                size = field_definition.size

        return size

    def __init__(self, definition_message=None, developer_fields=None, local_id: int = 0,
                 endian: Endian = Endian.LITTLE):
        super().__init__(name=DeviceInfoMessage.NAME,
                         global_id=DeviceInfoMessage.ID,
                         local_id=definition_message.local_id if definition_message else local_id,
                         endian=definition_message.endian if definition_message else endian,
                         definition_message=definition_message,
                         developer_fields=developer_fields,
                         fields=[
                             TimestampField(
                                 size=self.__get_field_size(definition_message, TimestampField.ID),
                                 growable=definition_message is None),
                             DeviceInfoDeviceIndexField(
                                 size=self.__get_field_size(definition_message, DeviceInfoDeviceIndexField.ID),
                                 growable=definition_message is None),
                             DeviceInfoDeviceTypeField(
                                 size=self.__get_field_size(definition_message, DeviceInfoDeviceTypeField.ID),
                                 growable=definition_message is None),
                             DeviceInfoManufacturerField(
                                 size=self.__get_field_size(definition_message, DeviceInfoManufacturerField.ID),
                                 growable=definition_message is None),
                             DeviceInfoSerialNumberField(
                                 size=self.__get_field_size(definition_message, DeviceInfoSerialNumberField.ID),
                                 growable=definition_message is None),
                             DeviceInfoProductField(
                                 size=self.__get_field_size(definition_message, DeviceInfoProductField.ID),
                                 growable=definition_message is None),
                             DeviceInfoSoftwareVersionField(
                                 size=self.__get_field_size(definition_message, DeviceInfoSoftwareVersionField.ID),
                                 growable=definition_message is None),
                             DeviceInfoHardwareVersionField(
                                 size=self.__get_field_size(definition_message, DeviceInfoHardwareVersionField.ID),
                                 growable=definition_message is None),
                             DeviceInfoCumOperatingTimeField(
                                 size=self.__get_field_size(definition_message, DeviceInfoCumOperatingTimeField.ID),
                                 growable=definition_message is None),
                             DeviceInfoBatteryVoltageField(
                                 size=self.__get_field_size(definition_message, DeviceInfoBatteryVoltageField.ID),
                                 growable=definition_message is None),
                             DeviceInfoBatteryStatusField(
                                 size=self.__get_field_size(definition_message, DeviceInfoBatteryStatusField.ID),
                                 growable=definition_message is None),
                             DeviceInfoSensorPositionField(
                                 size=self.__get_field_size(definition_message, DeviceInfoSensorPositionField.ID),
                                 growable=definition_message is None),
                             DeviceInfoDescriptorField(
                                 size=self.__get_field_size(definition_message, DeviceInfoDescriptorField.ID),
                                 growable=definition_message is None),
                             DeviceInfoAntTransmissionTypeField(
                                 size=self.__get_field_size(definition_message, DeviceInfoAntTransmissionTypeField.ID),
                                 growable=definition_message is None),
                             DeviceInfoAntDeviceNumberField(
                                 size=self.__get_field_size(definition_message, DeviceInfoAntDeviceNumberField.ID),
                                 growable=definition_message is None),
                             DeviceInfoAntNetworkField(
                                 size=self.__get_field_size(definition_message, DeviceInfoAntNetworkField.ID),
                                 growable=definition_message is None),
                             DeviceInfoSourceTypeField(
                                 size=self.__get_field_size(definition_message, DeviceInfoSourceTypeField.ID),
                                 growable=definition_message is None),
                             DeviceInfoProductNameField(
                                 size=self.__get_field_size(definition_message, DeviceInfoProductNameField.ID),
                                 growable=definition_message is None)
                         ])

        self.growable = self.definition_message is None

    @classmethod
    def from_bytes(cls, definition_message: DefinitionMessage, developer_fields: list[DeveloperField],
                   bytes_buffer: bytes, offset: int = 0):
        message = cls(definition_message=definition_message, developer_fields=developer_fields)
        message.read_from_bytes(bytes_buffer, offset)
        return message

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @property
    def timestamp(self) -> Optional[int]:
        field = self.get_field(TimestampField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @timestamp.setter
    def timestamp(self, value: int):
        field = self.get_field(TimestampField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def device_index(self) -> Optional[int]:
        field = self.get_field(DeviceInfoDeviceIndexField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @device_index.setter
    def device_index(self, value: int):
        field = self.get_field(DeviceInfoDeviceIndexField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def device_type(self) -> Optional[int]:
        field = self.get_field(DeviceInfoDeviceTypeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @device_type.setter
    def device_type(self, value: int):
        field = self.get_field(DeviceInfoDeviceTypeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def antplus_device_type(self) -> Optional[int]:
        field = self.get_field(DeviceInfoDeviceTypeField.ID)
        type_field = self.get_field(DeviceInfoSourceTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [1]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @antplus_device_type.setter
    def antplus_device_type(self, value: int):
        field = self.get_field(DeviceInfoDeviceTypeField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def ant_device_type(self) -> Optional[int]:
        field = self.get_field(DeviceInfoDeviceTypeField.ID)
        type_field = self.get_field(DeviceInfoSourceTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [0]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @ant_device_type.setter
    def ant_device_type(self, value: int):
        field = self.get_field(DeviceInfoDeviceTypeField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def manufacturer(self) -> Optional[int]:
        field = self.get_field(DeviceInfoManufacturerField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @manufacturer.setter
    def manufacturer(self, value: int):
        field = self.get_field(DeviceInfoManufacturerField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def serial_number(self) -> Optional[int]:
        field = self.get_field(DeviceInfoSerialNumberField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @serial_number.setter
    def serial_number(self, value: int):
        field = self.get_field(DeviceInfoSerialNumberField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def product(self) -> Optional[int]:
        field = self.get_field(DeviceInfoProductField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @product.setter
    def product(self, value: int):
        field = self.get_field(DeviceInfoProductField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def favero_product(self) -> Optional[int]:
        field = self.get_field(DeviceInfoProductField.ID)
        type_field = self.get_field(DeviceInfoManufacturerField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [263]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @favero_product.setter
    def favero_product(self, value: int):
        field = self.get_field(DeviceInfoProductField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def garmin_product(self) -> Optional[int]:
        field = self.get_field(DeviceInfoProductField.ID)
        type_field = self.get_field(DeviceInfoManufacturerField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [1, 15, 13, 89]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @garmin_product.setter
    def garmin_product(self, value: int):
        field = self.get_field(DeviceInfoProductField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def software_version(self) -> Optional[float]:
        field = self.get_field(DeviceInfoSoftwareVersionField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @software_version.setter
    def software_version(self, value: float):
        field = self.get_field(DeviceInfoSoftwareVersionField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def hardware_version(self) -> Optional[int]:
        field = self.get_field(DeviceInfoHardwareVersionField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @hardware_version.setter
    def hardware_version(self, value: int):
        field = self.get_field(DeviceInfoHardwareVersionField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def cum_operating_time(self) -> Optional[int]:
        field = self.get_field(DeviceInfoCumOperatingTimeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @cum_operating_time.setter
    def cum_operating_time(self, value: int):
        field = self.get_field(DeviceInfoCumOperatingTimeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def battery_voltage(self) -> Optional[float]:
        field = self.get_field(DeviceInfoBatteryVoltageField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @battery_voltage.setter
    def battery_voltage(self, value: float):
        field = self.get_field(DeviceInfoBatteryVoltageField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def battery_status(self) -> Optional[int]:
        field = self.get_field(DeviceInfoBatteryStatusField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @battery_status.setter
    def battery_status(self, value: int):
        field = self.get_field(DeviceInfoBatteryStatusField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def sensor_position(self) -> Optional[BodyLocation]:
        field = self.get_field(DeviceInfoSensorPositionField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @sensor_position.setter
    def sensor_position(self, value: BodyLocation):
        field = self.get_field(DeviceInfoSensorPositionField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def descriptor(self) -> Optional[str]:
        field = self.get_field(DeviceInfoDescriptorField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @descriptor.setter
    def descriptor(self, value: str):
        field = self.get_field(DeviceInfoDescriptorField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def ant_transmission_type(self) -> Optional[int]:
        field = self.get_field(DeviceInfoAntTransmissionTypeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @ant_transmission_type.setter
    def ant_transmission_type(self, value: int):
        field = self.get_field(DeviceInfoAntTransmissionTypeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def ant_device_number(self) -> Optional[int]:
        field = self.get_field(DeviceInfoAntDeviceNumberField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @ant_device_number.setter
    def ant_device_number(self, value: int):
        field = self.get_field(DeviceInfoAntDeviceNumberField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def ant_network(self) -> Optional[AntNetwork]:
        field = self.get_field(DeviceInfoAntNetworkField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @ant_network.setter
    def ant_network(self, value: AntNetwork):
        field = self.get_field(DeviceInfoAntNetworkField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def source_type(self) -> Optional[SourceType]:
        field = self.get_field(DeviceInfoSourceTypeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @source_type.setter
    def source_type(self, value: SourceType):
        field = self.get_field(DeviceInfoSourceTypeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def product_name(self) -> Optional[str]:
        field = self.get_field(DeviceInfoProductNameField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @product_name.setter
    def product_name(self, value: str):
        field = self.get_field(DeviceInfoProductNameField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)


class TimestampField(Field):
    ID = 253

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='timestamp',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=-631065600000,
            scale=0.001,
            size=size,
            units='ms',
            type_name='date_time',
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceInfoDeviceIndexField(Field):
    ID = 0

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='device_index',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceInfoDeviceTypeField(Field):
    ID = 1

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='device_type',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
                SubField(
                    name='antplus_device_type',
                    base_type=BaseType.UINT8,
                    scale=1,
                    offset=0,
                    reference_map={
                        DeviceInfoSourceTypeField.ID: [1]
                    }),
                SubField(
                    name='ant_device_type',
                    base_type=BaseType.UINT8,
                    scale=1,
                    offset=0,
                    reference_map={
                        DeviceInfoSourceTypeField.ID: [0]
                    })
            ]
        )


class DeviceInfoManufacturerField(Field):
    ID = 2

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='manufacturer',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceInfoSerialNumberField(Field):
    ID = 3

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='serial_number',
            field_id=self.ID,
            base_type=BaseType.UINT32Z,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceInfoProductField(Field):
    ID = 4

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='product',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
                SubField(
                    name='favero_product',
                    base_type=BaseType.UINT16,
                    scale=1,
                    offset=0,
                    reference_map={
                        DeviceInfoManufacturerField.ID: [263]
                    }),
                SubField(
                    name='garmin_product',
                    base_type=BaseType.UINT16,
                    scale=1,
                    offset=0,
                    reference_map={
                        DeviceInfoManufacturerField.ID: [1, 15, 13, 89]
                    })
            ]
        )


class DeviceInfoSoftwareVersionField(Field):
    ID = 5

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='software_version',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=100,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceInfoHardwareVersionField(Field):
    ID = 6

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='hardware_version',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceInfoCumOperatingTimeField(Field):
    ID = 7

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='cum_operating_time',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            units='s',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceInfoBatteryVoltageField(Field):
    ID = 10

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='battery_voltage',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=256,
            size=size,
            units='V',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceInfoBatteryStatusField(Field):
    ID = 11

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='battery_status',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceInfoSensorPositionField(Field):
    ID = 18

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='sensor_position',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceInfoDescriptorField(Field):
    ID = 19

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='descriptor',
            field_id=self.ID,
            base_type=BaseType.STRING,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceInfoAntTransmissionTypeField(Field):
    ID = 20

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='ant_transmission_type',
            field_id=self.ID,
            base_type=BaseType.UINT8Z,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceInfoAntDeviceNumberField(Field):
    ID = 21

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='ant_device_number',
            field_id=self.ID,
            base_type=BaseType.UINT16Z,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceInfoAntNetworkField(Field):
    ID = 22

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='ant_network',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceInfoSourceTypeField(Field):
    ID = 25

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='source_type',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceInfoProductNameField(Field):
    ID = 27

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='product_name',
            field_id=self.ID,
            base_type=BaseType.STRING,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )



# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list
from typing import Optional

from fit_tool.base_type import BaseType
from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.endian import Endian
from fit_tool.field import Field


class FieldDescriptionMessage(DataMessage):
    ID = 206
    NAME = 'field_description'

    @staticmethod
    def __get_field_size(definition_message: DefinitionMessage, field_id: int) -> int:
        size = 0
        if definition_message:
            field_definition = definition_message.get_field_definition(field_id)
            if field_definition:
                size = field_definition.size

        return size

    def __init__(self, definition_message=None, developer_fields=None, local_id: int = 0,
                 endian: Endian = Endian.LITTLE):
        super().__init__(name=FieldDescriptionMessage.NAME,
                         global_id=FieldDescriptionMessage.ID,
                         local_id=definition_message.local_id if definition_message else local_id,
                         endian=definition_message.endian if definition_message else endian,
                         definition_message=definition_message,
                         developer_fields=developer_fields,
                         fields=[
                             FieldDescriptionDeveloperDataIndexField(
                                 size=self.__get_field_size(definition_message,
                                                            FieldDescriptionDeveloperDataIndexField.ID),
                                 growable=definition_message is None),
                             FieldDescriptionFieldDefinitionNumberField(
                                 size=self.__get_field_size(definition_message,
                                                            FieldDescriptionFieldDefinitionNumberField.ID),
                                 growable=definition_message is None),
                             FieldDescriptionFitBaseTypeIdField(
                                 size=self.__get_field_size(definition_message, FieldDescriptionFitBaseTypeIdField.ID),
                                 growable=definition_message is None),
                             FieldDescriptionFieldNameField(
                                 size=self.__get_field_size(definition_message, FieldDescriptionFieldNameField.ID),
                                 growable=definition_message is None),
                             FieldDescriptionArrayField(
                                 size=self.__get_field_size(definition_message, FieldDescriptionArrayField.ID),
                                 growable=definition_message is None),
                             FieldDescriptionComponentsField(
                                 size=self.__get_field_size(definition_message, FieldDescriptionComponentsField.ID),
                                 growable=definition_message is None),
                             FieldDescriptionScaleField(
                                 size=self.__get_field_size(definition_message, FieldDescriptionScaleField.ID),
                                 growable=definition_message is None),
                             FieldDescriptionOffsetField(
                                 size=self.__get_field_size(definition_message, FieldDescriptionOffsetField.ID),
                                 growable=definition_message is None),
                             FieldDescriptionUnitsField(
                                 size=self.__get_field_size(definition_message, FieldDescriptionUnitsField.ID),
                                 growable=definition_message is None),
                             FieldDescriptionBitsField(
                                 size=self.__get_field_size(definition_message, FieldDescriptionBitsField.ID),
                                 growable=definition_message is None),
                             FieldDescriptionAccumulateField(
                                 size=self.__get_field_size(definition_message, FieldDescriptionAccumulateField.ID),
                                 growable=definition_message is None),
                             FieldDescriptionFitBaseUnitIdField(
                                 size=self.__get_field_size(definition_message, FieldDescriptionFitBaseUnitIdField.ID),
                                 growable=definition_message is None),
                             FieldDescriptionNativeMesgNumField(
                                 size=self.__get_field_size(definition_message, FieldDescriptionNativeMesgNumField.ID),
                                 growable=definition_message is None),
                             FieldDescriptionNativeFieldNumField(
                                 size=self.__get_field_size(definition_message, FieldDescriptionNativeFieldNumField.ID),
                                 growable=definition_message is None)
                         ])

        self.growable = self.definition_message is None

    @classmethod
    def from_bytes(cls, definition_message: DefinitionMessage, developer_fields: list[DeveloperField],
                   bytes_buffer: bytes, offset: int = 0):
        message = cls(definition_message=definition_message, developer_fields=developer_fields)
        message.read_from_bytes(bytes_buffer, offset)
        return message

    @property
    def developer_data_index(self) -> Optional[int]:
        field = self.get_field(FieldDescriptionDeveloperDataIndexField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @developer_data_index.setter
    def developer_data_index(self, value: int):
        field = self.get_field(FieldDescriptionDeveloperDataIndexField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def field_definition_number(self) -> Optional[int]:
        field = self.get_field(FieldDescriptionFieldDefinitionNumberField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @field_definition_number.setter
    def field_definition_number(self, value: int):
        field = self.get_field(FieldDescriptionFieldDefinitionNumberField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def fit_base_type_id(self) -> Optional[int]:
        field = self.get_field(FieldDescriptionFitBaseTypeIdField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @fit_base_type_id.setter
    def fit_base_type_id(self, value: int):
        field = self.get_field(FieldDescriptionFitBaseTypeIdField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def field_name(self) -> Optional[str]:
        field = self.get_field(FieldDescriptionFieldNameField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @field_name.setter
    def field_name(self, value: str):
        field = self.get_field(FieldDescriptionFieldNameField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def array(self) -> Optional[int]:
        field = self.get_field(FieldDescriptionArrayField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @array.setter
    def array(self, value: int):
        field = self.get_field(FieldDescriptionArrayField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def components(self) -> Optional[str]:
        field = self.get_field(FieldDescriptionComponentsField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @components.setter
    def components(self, value: str):
        field = self.get_field(FieldDescriptionComponentsField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def scale(self) -> Optional[int]:
        field = self.get_field(FieldDescriptionScaleField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @scale.setter
    def scale(self, value: int):
        field = self.get_field(FieldDescriptionScaleField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def offset(self) -> Optional[int]:
        field = self.get_field(FieldDescriptionOffsetField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @offset.setter
    def offset(self, value: int):
        field = self.get_field(FieldDescriptionOffsetField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def units(self) -> Optional[str]:
        field = self.get_field(FieldDescriptionUnitsField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @units.setter
    def units(self, value: str):
        field = self.get_field(FieldDescriptionUnitsField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def bits(self) -> Optional[str]:
        field = self.get_field(FieldDescriptionBitsField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @bits.setter
    def bits(self, value: str):
        field = self.get_field(FieldDescriptionBitsField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def accumulate(self) -> Optional[str]:
        field = self.get_field(FieldDescriptionAccumulateField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @accumulate.setter
    def accumulate(self, value: str):
        field = self.get_field(FieldDescriptionAccumulateField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def fit_base_unit_id(self) -> Optional[int]:
        field = self.get_field(FieldDescriptionFitBaseUnitIdField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @fit_base_unit_id.setter
    def fit_base_unit_id(self, value: int):
        field = self.get_field(FieldDescriptionFitBaseUnitIdField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def native_mesg_num(self) -> Optional[int]:
        field = self.get_field(FieldDescriptionNativeMesgNumField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @native_mesg_num.setter
    def native_mesg_num(self, value: int):
        field = self.get_field(FieldDescriptionNativeMesgNumField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def native_field_num(self) -> Optional[int]:
        field = self.get_field(FieldDescriptionNativeFieldNumField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @native_field_num.setter
    def native_field_num(self, value: int):
        field = self.get_field(FieldDescriptionNativeFieldNumField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)


class FieldDescriptionDeveloperDataIndexField(Field):
    ID = 0

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='developer_data_index',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class FieldDescriptionFieldDefinitionNumberField(Field):
    ID = 1

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='field_definition_number',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class FieldDescriptionFitBaseTypeIdField(Field):
    ID = 2

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='fit_base_type_id',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class FieldDescriptionFieldNameField(Field):
    ID = 3

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='field_name',
            field_id=self.ID,
            base_type=BaseType.STRING,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class FieldDescriptionArrayField(Field):
    ID = 4

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='array',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class FieldDescriptionComponentsField(Field):
    ID = 5

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='components',
            field_id=self.ID,
            base_type=BaseType.STRING,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class FieldDescriptionScaleField(Field):
    ID = 6

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='scale',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class FieldDescriptionOffsetField(Field):
    ID = 7

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='offset',
            field_id=self.ID,
            base_type=BaseType.SINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class FieldDescriptionUnitsField(Field):
    ID = 8

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='units',
            field_id=self.ID,
            base_type=BaseType.STRING,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class FieldDescriptionBitsField(Field):
    ID = 9

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='bits',
            field_id=self.ID,
            base_type=BaseType.STRING,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class FieldDescriptionAccumulateField(Field):
    ID = 10

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='accumulate',
            field_id=self.ID,
            base_type=BaseType.STRING,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class FieldDescriptionFitBaseUnitIdField(Field):
    ID = 13

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='fit_base_unit_id',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class FieldDescriptionNativeMesgNumField(Field):
    ID = 14

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='native_mesg_num',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class FieldDescriptionNativeFieldNumField(Field):
    ID = 15

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='native_field_num',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )
